using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.GlobalHelper;
using CMS.TreeEngine;

/// <summary>
/// Base page for the CMSDesk pages to apply global settings to the pages
/// </summary>
public abstract class CMSDeskPage : System.Web.UI.Page
{
    /// <summary>
    /// Generic body class - contains the base body css class
    /// </summary>
    protected string mBodyClass = "";


    public CMSDeskPage()
    {
        this.Load += new EventHandler(BasePage_Load);
    }
    
    protected void BasePage_Load(object sender, EventArgs e)
    {
        if (CultureHelper.IsUICultureRTL())
        {
            if (this.Header != null)
            {
                this.Header.Controls.Add(new LiteralControl("<style type=\"text/css\">* { direction: rtl; }</style>"));
                mBodyClass = "RTL";
            }
        }

        Response.Cache.SetNoStore();

        if ( CMSContext.CurrentSite == null )
        {
            Response.Redirect("~/CMSDesk/accessdenied.aspx");
        }

        CurrentUserInfo currentUser = CMSContext.CurrentUser;
        if (currentUser == null)
        {
            Response.Redirect("~/CMSDesk/accessdenied.aspx");
        }
        else if (! (currentUser.IsGlobalAdministrator || (currentUser.IsEditor && currentUser.IsInSite(CMSContext.CurrentSite.SiteName)) ))
        {
            Response.Redirect("~/CMSDesk/accessdenied.aspx");            
        }

        this.ClientScript.RegisterClientScriptBlock(typeof(string), "InfoScript", ScriptHelper.GetScript("function IsCMSDesk() { return true; }"));
    }


    /// <summary>
    /// Redirects the user to Access denied page
    /// </summary>
    /// <param name="resourceName">Resource name that failed</param>
    /// <param name="permissionName">Permission name that failed</param>
    public static void RedirectToAccessDenied(string resourceName, string permissionName)
    {        
        HttpContext.Current.Response.Redirect("~/CMSDesk/accessdenied.aspx?resource=" + resourceName + "&permission=" + permissionName);
    }


    /// <summary>
    /// Redirects the user to Access denied page
    /// </summary>
    /// <param name="nodeId">Node ID that failed</param>
    /// <param name="permissionName">Permission name that failed</param>
    public static void RedirectToAccessDenied(int nodeId, string permissionName)
    {
       HttpContext.Current.Response.Redirect("~/CMSDesk/accessdenied.aspx?nodeid=" + nodeId.ToString() + "&permission=" + permissionName);
    }

    /// <summary>
    /// Redirects the user to Access denied page
    /// </summary>
    /// <param name="message">Displayed access denied message.</param>
    public static void RedirectToAccessDenied(string message)
    {
        HttpContext.Current.Response.Redirect("~/CMSDesk/accessdenied.aspx?message=" + message);
    }

    /// <summary>Returns the absolute URL representation for the given relative URL</summary>
    /// <param name="relativeUrl">Relative URL to convert to absolute</param>
    public string AbsoluteUrl(string relativeUrl)
    {
        relativeUrl = ResolveUrl(relativeUrl);

        // Then add the base URL to the beginning of the URL where necessary
        if (relativeUrl.StartsWith("/"))
        {
            string webUrl = HttpContext.Current.Request.Url.AbsoluteUri.Substring(0, HttpContext.Current.Request.Url.AbsoluteUri.Length - HttpContext.Current.Request.Url.PathAndQuery.Length);
            if (!webUrl.EndsWith("/"))
            {
                webUrl += "/";
            }
            return webUrl + relativeUrl.TrimStart('/');
        }
        else
        {
            return relativeUrl;
        }
    }


    /// <summary>
    /// Signs the user out
    /// </summary>
    public void SignOut()
    {
        CMSContext.CurrentUser.ViewMode = ViewModeEnum.LiveSite;
        CMSContext.CurrentUser.MenuItemViewMode = ViewModeEnum.LiveSite;
        CMSContext.CurrentUser = null; 
        FormsAuthentication.SignOut();
        CookieHelper.Remove("RememberMeName");
    }


    /// <summary>
    /// Sets the current view mode based on the given node
    /// </summary>
    /// <param name="node">Document node</param>
    /// <param name="baseMode">Current base mode (edit/preview/livesite)</param>
    public ViewModeEnum UpdateViewMode(CMS.TreeEngine.TreeNode node, ViewModeEnum baseMode)
    {
        // Set mode if given
        string mode = "";

        // Get current mode
        ViewModeEnum currentMode = ViewModeEnum.Edit;
        ViewModeEnum currentMenuItemMode = ViewModeEnum.Edit;

        currentMode = CMSContext.CurrentUser.ViewMode;
        currentMenuItemMode = CMSContext.CurrentUser.MenuItemViewMode;

        ViewModeEnum newMode = currentMode;
        ViewModeEnum newMenuItemMode = currentMenuItemMode;

        // If mode within querystring, set mode
        if (Request.QueryString["mode"] != null)
        {
            mode = Request.QueryString["mode"].ToLower();
            if (TreePathUtils.IsMenuItemType(node.NodeClassName))
            {
                switch (mode)
                {
                    case "editform":
                        newMenuItemMode = ViewModeEnum.EditForm;
                        break;
                    case "edit":
                        newMenuItemMode = ViewModeEnum.Edit;
                        break;
                    case "design":
                        if (CMSContext.CurrentUser.IsGlobalAdministrator)
                        {
                            newMenuItemMode = ViewModeEnum.Design;
                        }
                        break;
                }
            }
            else
            {
                switch (mode)
                {
                    case "editform":
                        newMode = ViewModeEnum.EditForm;
                        break;
                    case "edit":
                        newMode = ViewModeEnum.Edit;
                        break;
                    case "design":
                        if (CMSContext.CurrentUser.IsGlobalAdministrator)
                        {
                            newMode = ViewModeEnum.Design;
                        }
                        break;
                }
            }
        }
        else
        {
            // Switch to the base mode if wrong mode
            switch (baseMode)
            {
                case ViewModeEnum.Edit:
                    // Get the new mode
                    if ((currentMode == ViewModeEnum.LiveSite) || (currentMode == ViewModeEnum.Preview))
                    {
                        newMode = ViewModeEnum.Edit;
                    }

                    // Get the new MenuItemMode
                    if ((currentMenuItemMode == ViewModeEnum.LiveSite) || (currentMenuItemMode == ViewModeEnum.Preview))
                    {
                        newMenuItemMode = ViewModeEnum.Edit;
                    }

                    break;
            }
        }

        // Set the mode
        CMSContext.CurrentUser.ViewMode = newMode;
        CMSContext.CurrentUser.MenuItemViewMode = newMenuItemMode;

        if (!TreePathUtils.IsMenuItemType(node.NodeClassName))
        {
            return newMode;
        }
        else
        {
            return newMenuItemMode;
        }
    }


    /// <summary>
    /// Determines whether current user is authorized to access cms.content
    /// </summary>
    /// <returns></returns>
    public static bool IsUserAuthorizedPerContent()
    {
        TreeProvider tp = new TreeProvider();
        bool userHasExploreTreePermission = false;
        bool userHasReadPermission = false;        
        
        SiteInfo currentSite = CMSContext.CurrentSite;
        if (currentSite != null)
        {            
            CMS.TreeEngine.TreeNode rootNode = tp.SelectSingleNode(CMSContext.CurrentSite.SiteName, "/", TreeProvider.ALL_CULTURES);
            if (rootNode != null)
            {
                int currentSiteRootId = rootNode.NodeID;

                // check 'exploretree' and 'read' permissions for cms.content
                userHasExploreTreePermission = (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(currentSiteRootId, NodePermissionsEnum.ExploreTree) == AuthorizationResultEnum.Allowed) ? true : false;
                userHasReadPermission = (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(currentSiteRootId, NodePermissionsEnum.Read) == AuthorizationResultEnum.Allowed) ? true : false;
            }
        }
        return (userHasExploreTreePermission && userHasReadPermission);
    }
}
