using System;
using System.Collections;
using System.Data;
using System.IO;
using System.Security.Permissions;
using System.Web;
using System.Web.Caching;
using System.Web.Hosting;

using CMS.SettingsProvider;
using CMS.PortalEngine;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.TreeEngine;

/// <summary>
/// Virtual path provider dynamicly loads content from virtual paths
/// </summary>
public class DbPathProvider : VirtualPathProvider
{
    /// <summary>
    /// Web parts virtual directory
    /// </summary>
    public const string VD_WEBPARTS = "webparts";

    /// <summary>
    /// Transformations virtual directory
    /// </summary>
    public const string VD_TRANSFORMATIONS = "transformations";

    /// <summary>
    /// Layouts virtual directory
    /// </summary>
    public const string VD_LAYOUTS = "layouts";

    /// <summary>
    /// CSS stylesheets virtual directory
    /// </summary>
    public const string VD_CSSSTYLESHEETS = "cssstylesheets";
    

    protected string mVirtualDirectory;

    /// <summary>
    /// Public property 'VirtualDirectory'.
    /// </summary>
    public string VirtualDirectory
    {
        get
        {
            return mVirtualDirectory;
        }
        set
        {
            mVirtualDirectory = value;
        }
    }


    /// <summary>
    /// Sets new virtual path provider when application starts.
    /// </summary>
    public static void AppInitialize()
    {
        HostingEnvironment.RegisterVirtualPathProvider(new DbPathProvider());
    }


    /// <summary>
    /// DbPathProvider constructor.
    /// </summary>
    public DbPathProvider()
        : base()
    {
    }


    /// <summary>
    /// Finds whether the given path contains virtual directory.
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <returns>True or false.</returns>
    private bool IsPathVirtual(string virtualPath)
    {
        string virtualDirectories = ";transformations;layouts;cssstylesheets;webparts;";

        this.VirtualDirectory = GetVirtDirectory(virtualPath);

        return virtualDirectories.Contains(";" + this.VirtualDirectory + ";");
    }


    

    /// <summary>
    /// Finds whether the file, which is part of given virtual path, exists
    /// on the virtual file system or on the normal file system.
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <returns>True or false.</returns>
    public override bool FileExists(string virtualPath)
    {
        if (IsPathVirtual(virtualPath))
        {
            string fileName = null;
            // Determine whether the file exists on the virtual file system.
            try
            {
                switch (this.VirtualDirectory)
                {
                    case VD_TRANSFORMATIONS:
                        // Check if transformation exists
                        if (virtualPath.Contains("---showeditdelete"))
                        {
                            virtualPath = virtualPath.Replace("---showeditdelete", "");
                        }

                        fileName = TransformationInfoProvider.GetTransformationName(virtualPath);
                        TransformationInfo transformation = TransformationInfoProvider.GetTransformation(fileName);
                        break;

                    case VD_LAYOUTS:
                        // Check if layout exists
                        fileName = LayoutInfoProvider.GetLayoutName(virtualPath);
                        LayoutInfo layout = LayoutInfoProvider.GetLayoutInfo(fileName);
                        break;

                    case VD_CSSSTYLESHEETS:
                        // Check if styleshet exists
                        fileName = CssStylesheetInfoProvider.GetCSSStylesheetName(virtualPath);
                        CssStylesheetInfo cssSI = CssStylesheetInfoProvider.GetCssStylesheetInfo(fileName);
                        break;

                    case VD_WEBPARTS:
                        // Do not check, loads dynamically within processing
                        break;
                }
                return true;
            }
            catch
            {
                return Previous.FileExists(virtualPath);
            }

        }
        else
        {
            // if not, the file is sought on the normal file system
            return Previous.FileExists(virtualPath);
        }
    }


    /// <summary>
    /// Finds whether the directory, which is part of given virtual path, exists
    /// on the virtual file system or on the normal file system.
    /// </summary>
    /// <param name="virtualDir"></param>
    /// <returns>True or false.</returns>
    public override bool DirectoryExists(string virtualDir)
    {
        if (IsPathVirtual(virtualDir))
        {
            return true;
        }
        else
        {
            return Previous.DirectoryExists(virtualDir);
        }
    }


    /// <summary>
    /// This method is used by the compilation system to obtain a VirtualFile instance to 
    /// work with a given virtual file path.
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <returns>Virtual File.</returns>
    public override VirtualFile GetFile(string virtualPath)
    {
        if ( IsPathVirtual(virtualPath) )
        {
            return new DbVirtualFile(virtualPath, this);
        }
        else
        {
            return Previous.GetFile(virtualPath);
        }
    }


    /// <summary>
    /// This method is used by the compilation system to obtain a VirtualDirectory 
    /// instance to work with a given virtual directory.
    /// </summary>
    /// <param name="virtualDir"></param>
    /// <returns>Virtual Directory.</returns>
    public override VirtualDirectory GetDirectory(string virtualDir)
    {
        if (IsPathVirtual(virtualDir))
        {
            return new DbVirtualDirectory(virtualDir, this);
        }
        else
        {
            return Previous.GetDirectory(virtualDir);
        }
    }


    /// <summary>
    /// Returns virtual directories from the virtual path
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <returns></returns>
    private string GetVirtDirectory(string virtualPath)
    {
        // Get local virtual path
        String checkPath = VirtualPathUtility.ToAppRelative(virtualPath);

        // Check the directory
        if ( checkPath.StartsWith(TransformationInfoProvider.TransformationsDirectory, StringComparison.InvariantCultureIgnoreCase) )
        {
            // Transformations
            return VD_TRANSFORMATIONS;
        }
        else if ( checkPath.StartsWith(LayoutInfoProvider.LayoutsDirectory, StringComparison.InvariantCultureIgnoreCase) )
        {
            // Layouts
            return VD_LAYOUTS;
        }
        else if ( checkPath.StartsWith(CssStylesheetInfoProvider.CSSStylesheetsDirectory, StringComparison.InvariantCultureIgnoreCase) )
        {
            // CSS Stylesheets
            return VD_CSSSTYLESHEETS;
        }
        else if ( checkPath.StartsWith(WebPartInfoProvider.VirtualWebPartsDirectory, StringComparison.InvariantCultureIgnoreCase) )
        {
            // Web parts
            return VD_WEBPARTS;
        }
        else
        {
            return "";
        }
    }


    /// <summary>
    /// Gets string content of the file, which is part of given transformated virtual path.
    /// </summary>
    /// <param name="vPath">Virtual path.</param>
    /// <param name="vDir">Virtual directory.</param>
    /// <returns>File content from hashtable 'transformations'.</returns>
    public string GetFileContent(string vPath, string vDir)
    {
        string content = null;

        string editDeleteButton = "";

        if ((vPath != null) && (vPath != ""))
        {
            switch (vDir)
            {
                case VD_TRANSFORMATIONS:
                    // Get transformation code
                    
                    if (vPath.Contains("---showeditdelete"))
                    {
                        vPath = vPath.Replace("---showeditdelete", "");
                        editDeleteButton = "<cc1:CMSEditModeButtonEditDelete runat=\"server\" id=\"btnEditDeleteAutoInsert\" Path='<%# Eval(\"NodeAliasPath\") %>' />";
                    }

                    TransformationInfo ti = TransformationInfoProvider.GetTransformation(vPath);
                    if (ti != null)
                    {
                        content = GetTransformationDirectives() + ti.TransformationCode + editDeleteButton;
                    }
                    else
                    {
                        throw new Exception("[DbPathProvider]: Transformation '" + vPath + "' not found!");
                    }
                    break;

                case VD_LAYOUTS:

                    // Trim the parameters (because of the caching
                    int paramIndex = vPath.IndexOf("---");
                    if (paramIndex > 0)
                    {
                        vPath = vPath.Substring(0, paramIndex);
                    }
                    
                    // Get layout code
                    LayoutInfo li = LayoutInfoProvider.GetLayoutInfo(vPath);
                    if (li != null)
                    {
                        content = GetLayoutDirectives() + li.LayoutCode;
                    }
                    else
                    {
                        throw new Exception("[DbPathProvider]: Layout '" + vPath + "' not found!");
                    }
                    break;

                case VD_CSSSTYLESHEETS:
                    // Get stylesheet code
                    CssStylesheetInfo cssi = CssStylesheetInfoProvider.GetCssStylesheetInfo(vPath);
                    if (cssi != null)
                    {
                        content = cssi.StylesheetText;
                    }
                    else
                    {
                        throw new Exception("[DbPathProvider]: CssStylesheet '" + vPath + "' not found!");
                    }
                    break;

                case VD_WEBPARTS:
                    // Process the path (parameters) - AliasPath/ZoneID/WebPartID
                    int lastSlash = vPath.LastIndexOf("/");
                    if (lastSlash >= 0)
                    {
                        // Web part ID
                        string webPartId = vPath.Substring(lastSlash + 1);
                        vPath = vPath.Substring(0, lastSlash);
                        // Zone ID
                        lastSlash = vPath.LastIndexOf("/");
                        string zoneId = vPath.Substring(lastSlash + 1);
                        string aliasPath = "/";
                        if (lastSlash >= 0)
                        {
                            aliasPath += vPath.Substring(0, lastSlash);
                        }

                        // Get the web part data
                        if ((webPartId != "") && (zoneId != "") && (aliasPath != ""))
                        {
                            // get pageinfo
                            PageInfo pi = PageInfoProvider.GetPageInfo(CMSContext.CurrentSite.SiteName, aliasPath, CMSContext.CurrentUser.PreferredCultureCode, null, CMSContext.CurrentSite.CombineWithDefaultCulture);
                            if (pi == null)
                            {
                                throw new Exception("[DbPathProvider.GetFileContent]: Page '" + aliasPath + "' not found.");
                            }
                            // get zone instance
                            WebPartZoneInstance zone = pi.PageTemplateInfo.GetZone(zoneId);
                            if (zone == null)
                            {
                                throw new Exception("[DbPathProvider.GetFileContent]: Zone '" + zoneId + "' not found.");
                            }
                            // get webpart instance
                            WebPartInstance webPart = zone.GetWebPart(webPartId);
                            if (webPart == null)
                            {
                                throw new Exception("[DbPathProvider.GetFileContent]: Web part '" + webPartId + "' not found.");
                            }

                            // Get web part
                            WebPartInfo part = WebPartInfoProvider.GetWebPartInfo(webPart.WebPartType);
                            if (part != null)
                            {
                                // Load the original web part file
                                string webPartFile = HttpContext.Current.Server.MapPath(WebPartInfoProvider.GetWebPartUrl(part));
                                TextReader reader = new StreamReader(webPartFile);
                                content = reader.ReadToEnd();
                                reader.Close();

                                if ((TreePathUtils.IsMenuItemType(pi.ClassName) && (CMSContext.CurrentUser.MenuItemViewMode != ViewModeEnum.Design)) || (!TreePathUtils.IsMenuItemType(pi.ClassName) && (CMSContext.CurrentUser.MenuItemViewMode != ViewModeEnum.Design)))
                                {
                                    // Add the additional code
                                    content += "\n" + webPart.GetValue("WebPartCode");
                                }
                            }
                            else
                            {
                                throw new Exception("[DbPathProvider.GetFileContent]: Web part '" + webPart.WebPartType + "' not found!");
                            }
                        }
                    }
                    break;

                default:
                    content = "";
                    break;
            }
        }

        return content;
    }


    /// <summary>
    /// Overriden method 'GetCacheDependency()'.
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <param name="virtualPathDependencies"></param>
    /// <param name="utcStart"></param>
    /// <returns>CacheDependency.</returns>
    public override CacheDependency GetCacheDependency(string virtualPath, System.Collections.IEnumerable virtualPathDependencies, DateTime utcStart)
    {
        if (IsPathVirtual(virtualPath))
        {
            return null;
        }
        else
        {
            return Previous.GetCacheDependency(virtualPath, virtualPathDependencies, utcStart);
        }
    }


    /// <summary>
    /// Overriden method 'GetFileHash'
    /// </summary>
    /// <param name="virtualPath"></param>
    /// <param name="virtualPathDependencies"></param>
    /// <returns></returns>
    public override string GetFileHash(string virtualPath, IEnumerable virtualPathDependencies)
    {
        return null;
    }


    /// <summary>
    /// Transformation directives are set before transformation code.
    /// </summary>
    /// <returns></returns>
    protected string GetTransformationDirectives()
    {
        // Get language code from web.config.
        string lang = CMS.GlobalHelper.DataHelper.GetNotEmpty(System.Configuration.ConfigurationManager.AppSettings["CMSProgrammingLanguage"], "C#");
        return "<%@ Control Language=\"" + lang + "\" AutoEventWireup=\"true\" Inherits=\"CMS.Controls.CMSAbstractTransformation\" %>" + "\n" + "<%@ Register TagPrefix=\"cc1\" Namespace=\"CMS.Controls\" Assembly=\"CMS.Controls\" %>" + "\n";
    }


    /// <summary>
    /// Layout directives are set before layout code.
    /// </summary>
    /// <returns></returns>
    protected string GetLayoutDirectives()
    {
        // Get language code from web.config.
        string lang = CMS.GlobalHelper.DataHelper.GetNotEmpty(System.Configuration.ConfigurationManager.AppSettings["CMSProgrammingLanguage"], "C#");
        return "<%@ Control Language=\"" + lang + "\" ClassName=\"Simple\" Inherits=\"CMS.PortalControls.CMSAbstractLayout\" %> \n"
             + "<%@ Register Assembly=\"CMS.PortalControls\" Namespace=\"CMS.PortalControls\" TagPrefix=\"cc1\" %> \n";
    }
}
